from ...mapper import Mapper
from ..loader import EnvVarLoader
from ...protos.physics_pb2 import Point
from typing import Tuple

def DefaultInitBundle() -> Tuple[EnvVarLoader, Mapper, Point]:
    defaultConfig = EnvVarLoader()

    defaultMapper = Mapper(10, 6, defaultConfig.get_bot_team_side())

    initialRegion = defaultMapper.get_region(DEFAULT_PLAYER_POSITIONS[defaultConfig.get_bot_number()]["Col"], DEFAULT_PLAYER_POSITIONS[defaultConfig.get_bot_number()]["Row"])

    defaultInitialPosition = initialRegion.get_center()

    return [ defaultConfig, defaultMapper, defaultInitialPosition]

DEFAULT_PLAYER_POSITIONS = {
    1: {"Col": 0, "Row": 0},
    2: {"Col": 1, "Row": 1},
    3: {"Col": 2, "Row": 2},
    4: {"Col": 2, "Row": 3},
    5: {"Col": 1, "Row": 4},
    6: {"Col": 3, "Row": 1},
    7: {"Col": 3, "Row": 2},
    8: {"Col": 3, "Row": 3},
    9: {"Col": 3, "Row": 4},
    10: {"Col": 4, "Row": 3},
    11: {"Col": 4, "Row": 2},
}



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Vector(_message.Message):
    __slots__ = ("x", "y")
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    x: float
    y: float
    def __init__(self, x: _Optional[float] = ..., y: _Optional[float] = ...) -> None: ...

class Point(_message.Message):
    __slots__ = ("x", "y")
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    x: int
    y: int
    def __init__(self, x: _Optional[int] = ..., y: _Optional[int] = ...) -> None: ...

class Velocity(_message.Message):
    __slots__ = ("direction", "speed")
    DIRECTION_FIELD_NUMBER: _ClassVar[int]
    SPEED_FIELD_NUMBER: _ClassVar[int]
    direction: Vector
    speed: float
    def __init__(self, direction: _Optional[_Union[Vector, _Mapping]] = ..., speed: _Optional[float] = ...) -> None: ...



import physics_pb2 as _physics_pb2
import server_pb2 as _server_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PauseResumeRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class NextTurnRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class NextOrderRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class BallProperties(_message.Message):
    __slots__ = ("position", "velocity", "holder")
    POSITION_FIELD_NUMBER: _ClassVar[int]
    VELOCITY_FIELD_NUMBER: _ClassVar[int]
    HOLDER_FIELD_NUMBER: _ClassVar[int]
    position: _physics_pb2.Point
    velocity: _physics_pb2.Velocity
    holder: _server_pb2.Player
    def __init__(self, position: _Optional[_Union[_physics_pb2.Point, _Mapping]] = ..., velocity: _Optional[_Union[_physics_pb2.Velocity, _Mapping]] = ..., holder: _Optional[_Union[_server_pb2.Player, _Mapping]] = ...) -> None: ...

class PlayerProperties(_message.Message):
    __slots__ = ("side", "number", "position", "velocity")
    SIDE_FIELD_NUMBER: _ClassVar[int]
    NUMBER_FIELD_NUMBER: _ClassVar[int]
    POSITION_FIELD_NUMBER: _ClassVar[int]
    VELOCITY_FIELD_NUMBER: _ClassVar[int]
    side: _server_pb2.Team.Side
    number: int
    position: _physics_pb2.Point
    velocity: _physics_pb2.Velocity
    def __init__(self, side: _Optional[_Union[_server_pb2.Team.Side, str]] = ..., number: _Optional[int] = ..., position: _Optional[_Union[_physics_pb2.Point, _Mapping]] = ..., velocity: _Optional[_Union[_physics_pb2.Velocity, _Mapping]] = ...) -> None: ...

class GameProperties(_message.Message):
    __slots__ = ("turn", "home_score", "away_score", "frame_interval", "shot_clock")
    TURN_FIELD_NUMBER: _ClassVar[int]
    HOME_SCORE_FIELD_NUMBER: _ClassVar[int]
    AWAY_SCORE_FIELD_NUMBER: _ClassVar[int]
    FRAME_INTERVAL_FIELD_NUMBER: _ClassVar[int]
    SHOT_CLOCK_FIELD_NUMBER: _ClassVar[int]
    turn: int
    home_score: int
    away_score: int
    frame_interval: int
    shot_clock: _server_pb2.ShotClock
    def __init__(self, turn: _Optional[int] = ..., home_score: _Optional[int] = ..., away_score: _Optional[int] = ..., frame_interval: _Optional[int] = ..., shot_clock: _Optional[_Union[_server_pb2.ShotClock, _Mapping]] = ...) -> None: ...

class CommandResponse(_message.Message):
    __slots__ = ("code", "game_snapshot", "details")
    class StatusCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        SUCCESS: _ClassVar[CommandResponse.StatusCode]
        INVALID_VALUE: _ClassVar[CommandResponse.StatusCode]
        DEADLINE_EXCEEDED: _ClassVar[CommandResponse.StatusCode]
        OTHER: _ClassVar[CommandResponse.StatusCode]
    SUCCESS: CommandResponse.StatusCode
    INVALID_VALUE: CommandResponse.StatusCode
    DEADLINE_EXCEEDED: CommandResponse.StatusCode
    OTHER: CommandResponse.StatusCode
    CODE_FIELD_NUMBER: _ClassVar[int]
    GAME_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    DETAILS_FIELD_NUMBER: _ClassVar[int]
    code: CommandResponse.StatusCode
    game_snapshot: _server_pb2.GameSnapshot
    details: str
    def __init__(self, code: _Optional[_Union[CommandResponse.StatusCode, str]] = ..., game_snapshot: _Optional[_Union[_server_pb2.GameSnapshot, _Mapping]] = ..., details: _Optional[str] = ...) -> None: ...

class ResumeListeningRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ResumeListeningResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ResetPlayerPositionsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ResetGameRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ResetPlayerPositionsResponse(_message.Message):
    __slots__ = ("game_snapshot",)
    GAME_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    game_snapshot: _server_pb2.GameSnapshot
    def __init__(self, game_snapshot: _Optional[_Union[_server_pb2.GameSnapshot, _Mapping]] = ...) -> None: ...

class GameSnapshotRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GameSnapshotResponse(_message.Message):
    __slots__ = ("game_snapshot",)
    GAME_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    game_snapshot: _server_pb2.GameSnapshot
    def __init__(self, game_snapshot: _Optional[_Union[_server_pb2.GameSnapshot, _Mapping]] = ...) -> None: ...



from .game_snapshot_inspector import GameSnapshotInspector
from ..protos.server_pb2 import Team

class PlayerState(object):
    """
     Represents various states that a player can be in during a game.

     Attributes:
         SUPPORTING (int): The player does not hold the ball, but the holder is a teammate
         HOLDING_THE_BALL (int): The player is holding the ball.
         DEFENDING (int): The ball holder is an opponent player
         DISPUTING_THE_BALL (int): No one is holding the ball

     Methods:
         None
     """
    SUPPORTING = 0
    HOLDING_THE_BALL = 1
    DEFENDING = 2
    DISPUTING_THE_BALL = 3


PLAYER_STATE = PlayerState()

def define_state(inspector: GameSnapshotInspector, player_number: int, side: Team.Side) -> PLAYER_STATE:
    if not inspector or not inspector.get_ball():
        raise AttributeError(
            'invalid snapshot state - cannot define player state')


    me = inspector.get_me()
    if me is None:
        raise AttributeError(
            'could not find the bot in the snapshot - cannot define player state')

    ball_holder = inspector.get_ball_holder()

    if ball_holder.number == 0:
        return PLAYER_STATE.DISPUTING_THE_BALL

    if ball_holder.team_side == side:
        if ball_holder.number == player_number:
            return PLAYER_STATE.HOLDING_THE_BALL

        return PLAYER_STATE.SUPPORTING

    return PLAYER_STATE.DEFENDING



import server_pb2 as _server_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class WatcherRequest(_message.Message):
    __slots__ = ("uuid",)
    UUID_FIELD_NUMBER: _ClassVar[int]
    uuid: str
    def __init__(self, uuid: _Optional[str] = ...) -> None: ...

class StartRequest(_message.Message):
    __slots__ = ("watcher_uuid",)
    WATCHER_UUID_FIELD_NUMBER: _ClassVar[int]
    watcher_uuid: str
    def __init__(self, watcher_uuid: _Optional[str] = ...) -> None: ...

class GameEvent(_message.Message):
    __slots__ = ("game_snapshot", "new_player", "lost_player", "state_change", "goal", "game_over", "breakpoint", "debug_released")
    GAME_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    NEW_PLAYER_FIELD_NUMBER: _ClassVar[int]
    LOST_PLAYER_FIELD_NUMBER: _ClassVar[int]
    STATE_CHANGE_FIELD_NUMBER: _ClassVar[int]
    GOAL_FIELD_NUMBER: _ClassVar[int]
    GAME_OVER_FIELD_NUMBER: _ClassVar[int]
    BREAKPOINT_FIELD_NUMBER: _ClassVar[int]
    DEBUG_RELEASED_FIELD_NUMBER: _ClassVar[int]
    game_snapshot: _server_pb2.GameSnapshot
    new_player: EventNewPlayer
    lost_player: EventLostPlayer
    state_change: EventStateChange
    goal: EventGoal
    game_over: EventGameOver
    breakpoint: EventDebugBreakpoint
    debug_released: EventDebugReleased
    def __init__(self, game_snapshot: _Optional[_Union[_server_pb2.GameSnapshot, _Mapping]] = ..., new_player: _Optional[_Union[EventNewPlayer, _Mapping]] = ..., lost_player: _Optional[_Union[EventLostPlayer, _Mapping]] = ..., state_change: _Optional[_Union[EventStateChange, _Mapping]] = ..., goal: _Optional[_Union[EventGoal, _Mapping]] = ..., game_over: _Optional[_Union[EventGameOver, _Mapping]] = ..., breakpoint: _Optional[_Union[EventDebugBreakpoint, _Mapping]] = ..., debug_released: _Optional[_Union[EventDebugReleased, _Mapping]] = ...) -> None: ...

class GameSetup(_message.Message):
    __slots__ = ("protocol_version", "dev_mode", "start_mode", "listening_mode", "listening_duration", "game_duration", "home_team", "away_team")
    class StartingMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        NO_WAIT: _ClassVar[GameSetup.StartingMode]
        WAIT: _ClassVar[GameSetup.StartingMode]
    NO_WAIT: GameSetup.StartingMode
    WAIT: GameSetup.StartingMode
    class ListeningMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        TIMER: _ClassVar[GameSetup.ListeningMode]
        RUSH: _ClassVar[GameSetup.ListeningMode]
        REMOTE: _ClassVar[GameSetup.ListeningMode]
    TIMER: GameSetup.ListeningMode
    RUSH: GameSetup.ListeningMode
    REMOTE: GameSetup.ListeningMode
    PROTOCOL_VERSION_FIELD_NUMBER: _ClassVar[int]
    DEV_MODE_FIELD_NUMBER: _ClassVar[int]
    START_MODE_FIELD_NUMBER: _ClassVar[int]
    LISTENING_MODE_FIELD_NUMBER: _ClassVar[int]
    LISTENING_DURATION_FIELD_NUMBER: _ClassVar[int]
    GAME_DURATION_FIELD_NUMBER: _ClassVar[int]
    HOME_TEAM_FIELD_NUMBER: _ClassVar[int]
    AWAY_TEAM_FIELD_NUMBER: _ClassVar[int]
    protocol_version: str
    dev_mode: bool
    start_mode: GameSetup.StartingMode
    listening_mode: GameSetup.ListeningMode
    listening_duration: int
    game_duration: int
    home_team: TeamSettings
    away_team: TeamSettings
    def __init__(self, protocol_version: _Optional[str] = ..., dev_mode: bool = ..., start_mode: _Optional[_Union[GameSetup.StartingMode, str]] = ..., listening_mode: _Optional[_Union[GameSetup.ListeningMode, str]] = ..., listening_duration: _Optional[int] = ..., game_duration: _Optional[int] = ..., home_team: _Optional[_Union[TeamSettings, _Mapping]] = ..., away_team: _Optional[_Union[TeamSettings, _Mapping]] = ...) -> None: ...

class TeamSettings(_message.Message):
    __slots__ = ("name", "avatar", "colors")
    NAME_FIELD_NUMBER: _ClassVar[int]
    AVATAR_FIELD_NUMBER: _ClassVar[int]
    COLORS_FIELD_NUMBER: _ClassVar[int]
    name: str
    avatar: str
    colors: TeamColors
    def __init__(self, name: _Optional[str] = ..., avatar: _Optional[str] = ..., colors: _Optional[_Union[TeamColors, _Mapping]] = ...) -> None: ...

class TeamColors(_message.Message):
    __slots__ = ("primary", "secondary")
    PRIMARY_FIELD_NUMBER: _ClassVar[int]
    SECONDARY_FIELD_NUMBER: _ClassVar[int]
    primary: TeamColor
    secondary: TeamColor
    def __init__(self, primary: _Optional[_Union[TeamColor, _Mapping]] = ..., secondary: _Optional[_Union[TeamColor, _Mapping]] = ...) -> None: ...

class TeamColor(_message.Message):
    __slots__ = ("red", "green", "blue")
    RED_FIELD_NUMBER: _ClassVar[int]
    GREEN_FIELD_NUMBER: _ClassVar[int]
    BLUE_FIELD_NUMBER: _ClassVar[int]
    red: int
    green: int
    blue: int
    def __init__(self, red: _Optional[int] = ..., green: _Optional[int] = ..., blue: _Optional[int] = ...) -> None: ...

class EventNewPlayer(_message.Message):
    __slots__ = ("player",)
    PLAYER_FIELD_NUMBER: _ClassVar[int]
    player: _server_pb2.Player
    def __init__(self, player: _Optional[_Union[_server_pb2.Player, _Mapping]] = ...) -> None: ...

class EventLostPlayer(_message.Message):
    __slots__ = ("player",)
    PLAYER_FIELD_NUMBER: _ClassVar[int]
    player: _server_pb2.Player
    def __init__(self, player: _Optional[_Union[_server_pb2.Player, _Mapping]] = ...) -> None: ...

class EventStateChange(_message.Message):
    __slots__ = ("previous_state", "new_state")
    PREVIOUS_STATE_FIELD_NUMBER: _ClassVar[int]
    NEW_STATE_FIELD_NUMBER: _ClassVar[int]
    previous_state: _server_pb2.GameSnapshot.State
    new_state: _server_pb2.GameSnapshot.State
    def __init__(self, previous_state: _Optional[_Union[_server_pb2.GameSnapshot.State, str]] = ..., new_state: _Optional[_Union[_server_pb2.GameSnapshot.State, str]] = ...) -> None: ...

class EventGoal(_message.Message):
    __slots__ = ("side",)
    SIDE_FIELD_NUMBER: _ClassVar[int]
    side: _server_pb2.Team.Side
    def __init__(self, side: _Optional[_Union[_server_pb2.Team.Side, str]] = ...) -> None: ...

class EventGameOver(_message.Message):
    __slots__ = ("reason", "blame")
    class EndingReason(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        TIME_IS_OVER: _ClassVar[EventGameOver.EndingReason]
        WAITING_EXPIRED: _ClassVar[EventGameOver.EndingReason]
        NO_ENOUGH_PLAYER: _ClassVar[EventGameOver.EndingReason]
        EXTERNAL_REQUEST: _ClassVar[EventGameOver.EndingReason]
        KNOCKOUT: _ClassVar[EventGameOver.EndingReason]
    TIME_IS_OVER: EventGameOver.EndingReason
    WAITING_EXPIRED: EventGameOver.EndingReason
    NO_ENOUGH_PLAYER: EventGameOver.EndingReason
    EXTERNAL_REQUEST: EventGameOver.EndingReason
    KNOCKOUT: EventGameOver.EndingReason
    class BlameStop(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        NORMAL: _ClassVar[EventGameOver.BlameStop]
        BOTH_TEAMS: _ClassVar[EventGameOver.BlameStop]
        HOME_TEAM: _ClassVar[EventGameOver.BlameStop]
        AWAY_TEAM: _ClassVar[EventGameOver.BlameStop]
    NORMAL: EventGameOver.BlameStop
    BOTH_TEAMS: EventGameOver.BlameStop
    HOME_TEAM: EventGameOver.BlameStop
    AWAY_TEAM: EventGameOver.BlameStop
    REASON_FIELD_NUMBER: _ClassVar[int]
    BLAME_FIELD_NUMBER: _ClassVar[int]
    reason: EventGameOver.EndingReason
    blame: EventGameOver.BlameStop
    def __init__(self, reason: _Optional[_Union[EventGameOver.EndingReason, str]] = ..., blame: _Optional[_Union[EventGameOver.BlameStop, str]] = ...) -> None: ...

class EventDebugBreakpoint(_message.Message):
    __slots__ = ("breakpoint",)
    class Breakpoint(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        ORDERS: _ClassVar[EventDebugBreakpoint.Breakpoint]
        TURN: _ClassVar[EventDebugBreakpoint.Breakpoint]
    ORDERS: EventDebugBreakpoint.Breakpoint
    TURN: EventDebugBreakpoint.Breakpoint
    BREAKPOINT_FIELD_NUMBER: _ClassVar[int]
    breakpoint: EventDebugBreakpoint.Breakpoint
    def __init__(self, breakpoint: _Optional[_Union[EventDebugBreakpoint.Breakpoint, str]] = ...) -> None: ...

class EventDebugReleased(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



from src.lugo4py.protos.server_pb2 import GameSnapshot, Player, Team


def get_ball_holder(snapshot: GameSnapshot):
    holder = snapshot.ball.holder
    return holder if holder is not None else None

def is_ball_holder(snapshot: GameSnapshot, player: Player):
    holder = snapshot.ball.holder
    return holder is not None and holder.team_side == player.team_side and holder.number == player.number

def get_team(snapshot: GameSnapshot, side: Team.Side):
    if side == Team.Side.HOME:
        return snapshot.home_team
    else: return snapshot.away_team


def get_player(snapshot: GameSnapshot, side: Team.Side, number: int):
    team = get_team(snapshot, side)
    if team:
        for current_player in team.players:
            if current_player.number == number:
                return current_player
    return None

def get_opponent_side(side: Team.Side):
    return Team.Side.AWAY if side == Team.Side.HOME else Team.Side.HOME





import physics_pb2 as _physics_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class JoinRequest(_message.Message):
    __slots__ = ("token", "protocol_version", "team_side", "number", "init_position")
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    PROTOCOL_VERSION_FIELD_NUMBER: _ClassVar[int]
    TEAM_SIDE_FIELD_NUMBER: _ClassVar[int]
    NUMBER_FIELD_NUMBER: _ClassVar[int]
    INIT_POSITION_FIELD_NUMBER: _ClassVar[int]
    token: str
    protocol_version: str
    team_side: Team.Side
    number: int
    init_position: _physics_pb2.Point
    def __init__(self, token: _Optional[str] = ..., protocol_version: _Optional[str] = ..., team_side: _Optional[_Union[Team.Side, str]] = ..., number: _Optional[int] = ..., init_position: _Optional[_Union[_physics_pb2.Point, _Mapping]] = ...) -> None: ...

class GameSnapshot(_message.Message):
    __slots__ = ("state", "turn", "home_team", "away_team", "ball", "turns_ball_in_goal_zone", "shot_clock")
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        WAITING: _ClassVar[GameSnapshot.State]
        GET_READY: _ClassVar[GameSnapshot.State]
        LISTENING: _ClassVar[GameSnapshot.State]
        PLAYING: _ClassVar[GameSnapshot.State]
        SHIFTING: _ClassVar[GameSnapshot.State]
        OVER: _ClassVar[GameSnapshot.State]
    WAITING: GameSnapshot.State
    GET_READY: GameSnapshot.State
    LISTENING: GameSnapshot.State
    PLAYING: GameSnapshot.State
    SHIFTING: GameSnapshot.State
    OVER: GameSnapshot.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    TURN_FIELD_NUMBER: _ClassVar[int]
    HOME_TEAM_FIELD_NUMBER: _ClassVar[int]
    AWAY_TEAM_FIELD_NUMBER: _ClassVar[int]
    BALL_FIELD_NUMBER: _ClassVar[int]
    TURNS_BALL_IN_GOAL_ZONE_FIELD_NUMBER: _ClassVar[int]
    SHOT_CLOCK_FIELD_NUMBER: _ClassVar[int]
    state: GameSnapshot.State
    turn: int
    home_team: Team
    away_team: Team
    ball: Ball
    turns_ball_in_goal_zone: int
    shot_clock: ShotClock
    def __init__(self, state: _Optional[_Union[GameSnapshot.State, str]] = ..., turn: _Optional[int] = ..., home_team: _Optional[_Union[Team, _Mapping]] = ..., away_team: _Optional[_Union[Team, _Mapping]] = ..., ball: _Optional[_Union[Ball, _Mapping]] = ..., turns_ball_in_goal_zone: _Optional[int] = ..., shot_clock: _Optional[_Union[ShotClock, _Mapping]] = ...) -> None: ...

class Team(_message.Message):
    __slots__ = ("players", "name", "score", "side")
    class Side(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        HOME: _ClassVar[Team.Side]
        AWAY: _ClassVar[Team.Side]
    HOME: Team.Side
    AWAY: Team.Side
    PLAYERS_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    SCORE_FIELD_NUMBER: _ClassVar[int]
    SIDE_FIELD_NUMBER: _ClassVar[int]
    players: _containers.RepeatedCompositeFieldContainer[Player]
    name: str
    score: int
    side: Team.Side
    def __init__(self, players: _Optional[_Iterable[_Union[Player, _Mapping]]] = ..., name: _Optional[str] = ..., score: _Optional[int] = ..., side: _Optional[_Union[Team.Side, str]] = ...) -> None: ...

class ShotClock(_message.Message):
    __slots__ = ("team_side", "remaining_turns")
    TEAM_SIDE_FIELD_NUMBER: _ClassVar[int]
    REMAINING_TURNS_FIELD_NUMBER: _ClassVar[int]
    team_side: Team.Side
    remaining_turns: int
    def __init__(self, team_side: _Optional[_Union[Team.Side, str]] = ..., remaining_turns: _Optional[int] = ...) -> None: ...

class Player(_message.Message):
    __slots__ = ("number", "position", "velocity", "team_side", "init_position", "is_jumping")
    NUMBER_FIELD_NUMBER: _ClassVar[int]
    POSITION_FIELD_NUMBER: _ClassVar[int]
    VELOCITY_FIELD_NUMBER: _ClassVar[int]
    TEAM_SIDE_FIELD_NUMBER: _ClassVar[int]
    INIT_POSITION_FIELD_NUMBER: _ClassVar[int]
    IS_JUMPING_FIELD_NUMBER: _ClassVar[int]
    number: int
    position: _physics_pb2.Point
    velocity: _physics_pb2.Velocity
    team_side: Team.Side
    init_position: _physics_pb2.Point
    is_jumping: bool
    def __init__(self, number: _Optional[int] = ..., position: _Optional[_Union[_physics_pb2.Point, _Mapping]] = ..., velocity: _Optional[_Union[_physics_pb2.Velocity, _Mapping]] = ..., team_side: _Optional[_Union[Team.Side, str]] = ..., init_position: _Optional[_Union[_physics_pb2.Point, _Mapping]] = ..., is_jumping: bool = ...) -> None: ...

class Ball(_message.Message):
    __slots__ = ("position", "velocity", "holder")
    POSITION_FIELD_NUMBER: _ClassVar[int]
    VELOCITY_FIELD_NUMBER: _ClassVar[int]
    HOLDER_FIELD_NUMBER: _ClassVar[int]
    position: _physics_pb2.Point
    velocity: _physics_pb2.Velocity
    holder: Player
    def __init__(self, position: _Optional[_Union[_physics_pb2.Point, _Mapping]] = ..., velocity: _Optional[_Union[_physics_pb2.Velocity, _Mapping]] = ..., holder: _Optional[_Union[Player, _Mapping]] = ...) -> None: ...

class OrderResponse(_message.Message):
    __slots__ = ("code", "details")
    class StatusCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        SUCCESS: _ClassVar[OrderResponse.StatusCode]
        UNKNOWN_PLAYER: _ClassVar[OrderResponse.StatusCode]
        NOT_LISTENING: _ClassVar[OrderResponse.StatusCode]
        WRONG_TURN: _ClassVar[OrderResponse.StatusCode]
        OTHER: _ClassVar[OrderResponse.StatusCode]
    SUCCESS: OrderResponse.StatusCode
    UNKNOWN_PLAYER: OrderResponse.StatusCode
    NOT_LISTENING: OrderResponse.StatusCode
    WRONG_TURN: OrderResponse.StatusCode
    OTHER: OrderResponse.StatusCode
    CODE_FIELD_NUMBER: _ClassVar[int]
    DETAILS_FIELD_NUMBER: _ClassVar[int]
    code: OrderResponse.StatusCode
    details: str
    def __init__(self, code: _Optional[_Union[OrderResponse.StatusCode, str]] = ..., details: _Optional[str] = ...) -> None: ...

class OrderSet(_message.Message):
    __slots__ = ("turn", "orders", "debug_message")
    TURN_FIELD_NUMBER: _ClassVar[int]
    ORDERS_FIELD_NUMBER: _ClassVar[int]
    DEBUG_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    turn: int
    orders: _containers.RepeatedCompositeFieldContainer[Order]
    debug_message: str
    def __init__(self, turn: _Optional[int] = ..., orders: _Optional[_Iterable[_Union[Order, _Mapping]]] = ..., debug_message: _Optional[str] = ...) -> None: ...

class Order(_message.Message):
    __slots__ = ("move", "catch", "kick", "jump")
    MOVE_FIELD_NUMBER: _ClassVar[int]
    CATCH_FIELD_NUMBER: _ClassVar[int]
    KICK_FIELD_NUMBER: _ClassVar[int]
    JUMP_FIELD_NUMBER: _ClassVar[int]
    move: Move
    catch: Catch
    kick: Kick
    jump: Jump
    def __init__(self, move: _Optional[_Union[Move, _Mapping]] = ..., catch: _Optional[_Union[Catch, _Mapping]] = ..., kick: _Optional[_Union[Kick, _Mapping]] = ..., jump: _Optional[_Union[Jump, _Mapping]] = ...) -> None: ...

class Move(_message.Message):
    __slots__ = ("velocity",)
    VELOCITY_FIELD_NUMBER: _ClassVar[int]
    velocity: _physics_pb2.Velocity
    def __init__(self, velocity: _Optional[_Union[_physics_pb2.Velocity, _Mapping]] = ...) -> None: ...

class Catch(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class Kick(_message.Message):
    __slots__ = ("velocity",)
    VELOCITY_FIELD_NUMBER: _ClassVar[int]
    velocity: _physics_pb2.Velocity
    def __init__(self, velocity: _Optional[_Union[_physics_pb2.Velocity, _Mapping]] = ...) -> None: ...

class Jump(_message.Message):
    __slots__ = ("velocity",)
    VELOCITY_FIELD_NUMBER: _ClassVar[int]
    velocity: _physics_pb2.Velocity
    def __init__(self, velocity: _Optional[_Union[_physics_pb2.Velocity, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rl_assistant.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import server_pb2 as server__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12rl_assistant.proto\x12\x04lugo\x1a\x0cserver.proto\"\x11\n\x0fRLSessionConfig\"}\n\x17PlayerOrdersOnRLSession\x12\"\n\tteam_side\x18\x01 \x01(\x0e\x32\x0f.lugo.Team.Side\x12\x0e\n\x06number\x18\x02 \x01(\r\x12\x11\n\tbehaviour\x18\x03 \x01(\t\x12\x1b\n\x06orders\x18\x04 \x03(\x0b\x32\x0b.lugo.Order\"a\n\rPlayersOrders\x12\x19\n\x11\x64\x65\x66\x61ult_behaviour\x18\x01 \x01(\t\x12\x35\n\x0eplayers_orders\x18\x02 \x03(\x0b\x32\x1d.lugo.PlayerOrdersOnRLSession\"\x8c\x01\n\x0bTurnOutcome\x12)\n\rgame_snapshot\x18\x01 \x01(\x0b\x32\x12.lugo.GameSnapshot\x12\x15\n\rscore_changed\x18\x02 \x01(\x08\x12\x1a\n\x12shot_clock_expired\x18\x03 \x01(\x08\x12\x1f\n\x17goal_zone_timer_expired\x18\x04 \x01(\x08\"\x0f\n\rRLResetConfig2\xbb\x01\n\x0bRLAssistant\x12;\n\x0cStartSession\x12\x15.lugo.RLSessionConfig\x1a\x12.lugo.GameSnapshot0\x01\x12;\n\x11SendPlayersOrders\x12\x13.lugo.PlayersOrders\x1a\x11.lugo.TurnOutcome\x12\x32\n\x08ResetEnv\x12\x13.lugo.RLResetConfig\x1a\x11.lugo.TurnOutcomeB#Z!github.com/lugobots/lugo4go/protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rl_assistant_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z!github.com/lugobots/lugo4go/proto'
  _RLSESSIONCONFIG._serialized_start=42
  _RLSESSIONCONFIG._serialized_end=59
  _PLAYERORDERSONRLSESSION._serialized_start=61
  _PLAYERORDERSONRLSESSION._serialized_end=186
  _PLAYERSORDERS._serialized_start=188
  _PLAYERSORDERS._serialized_end=285
  _TURNOUTCOME._serialized_start=288
  _TURNOUTCOME._serialized_end=428
  _RLRESETCONFIG._serialized_start=430
  _RLRESETCONFIG._serialized_end=445
  _RLASSISTANT._serialized_start=448
  _RLASSISTANT._serialized_end=635
# @@protoc_insertion_point(module_scope)



import server_pb2 as _server_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class RLSessionConfig(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class PlayerOrdersOnRLSession(_message.Message):
    __slots__ = ("team_side", "number", "behaviour", "orders")
    TEAM_SIDE_FIELD_NUMBER: _ClassVar[int]
    NUMBER_FIELD_NUMBER: _ClassVar[int]
    BEHAVIOUR_FIELD_NUMBER: _ClassVar[int]
    ORDERS_FIELD_NUMBER: _ClassVar[int]
    team_side: _server_pb2.Team.Side
    number: int
    behaviour: str
    orders: _containers.RepeatedCompositeFieldContainer[_server_pb2.Order]
    def __init__(self, team_side: _Optional[_Union[_server_pb2.Team.Side, str]] = ..., number: _Optional[int] = ..., behaviour: _Optional[str] = ..., orders: _Optional[_Iterable[_Union[_server_pb2.Order, _Mapping]]] = ...) -> None: ...

class PlayersOrders(_message.Message):
    __slots__ = ("default_behaviour", "players_orders")
    DEFAULT_BEHAVIOUR_FIELD_NUMBER: _ClassVar[int]
    PLAYERS_ORDERS_FIELD_NUMBER: _ClassVar[int]
    default_behaviour: str
    players_orders: _containers.RepeatedCompositeFieldContainer[PlayerOrdersOnRLSession]
    def __init__(self, default_behaviour: _Optional[str] = ..., players_orders: _Optional[_Iterable[_Union[PlayerOrdersOnRLSession, _Mapping]]] = ...) -> None: ...

class TurnOutcome(_message.Message):
    __slots__ = ("game_snapshot", "score_changed", "shot_clock_expired", "goal_zone_timer_expired")
    GAME_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    SCORE_CHANGED_FIELD_NUMBER: _ClassVar[int]
    SHOT_CLOCK_EXPIRED_FIELD_NUMBER: _ClassVar[int]
    GOAL_ZONE_TIMER_EXPIRED_FIELD_NUMBER: _ClassVar[int]
    game_snapshot: _server_pb2.GameSnapshot
    score_changed: bool
    shot_clock_expired: bool
    goal_zone_timer_expired: bool
    def __init__(self, game_snapshot: _Optional[_Union[_server_pb2.GameSnapshot, _Mapping]] = ..., score_changed: bool = ..., shot_clock_expired: bool = ..., goal_zone_timer_expired: bool = ...) -> None: ...

class RLResetConfig(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: server.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import physics_pb2 as physics__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cserver.proto\x12\x04lugo\x1a\rphysics.proto\"\x8e\x01\n\x0bJoinRequest\x12\r\n\x05token\x18\x01 \x01(\t\x12\x18\n\x10protocol_version\x18\x02 \x01(\t\x12\"\n\tteam_side\x18\x03 \x01(\x0e\x32\x0f.lugo.Team.Side\x12\x0e\n\x06number\x18\x04 \x01(\r\x12\"\n\rinit_position\x18\x05 \x01(\x0b\x32\x0b.lugo.Point\"\xf4\x02\n\x0cGameSnapshot\x12\'\n\x05state\x18\x01 \x01(\x0e\x32\x18.lugo.GameSnapshot.State\x12\x0c\n\x04turn\x18\x02 \x01(\r\x12\x1d\n\thome_team\x18\x03 \x01(\x0b\x32\n.lugo.Team\x12\x1d\n\taway_team\x18\x04 \x01(\x0b\x32\n.lugo.Team\x12\x18\n\x04\x62\x61ll\x18\x05 \x01(\x0b\x32\n.lugo.Ball\x12\x1f\n\x17turns_ball_in_goal_zone\x18\x06 \x01(\r\x12#\n\nshot_clock\x18\x07 \x01(\x0b\x32\x0f.lugo.ShotClock\"\x8e\x01\n\x05State\x12\x0b\n\x07WAITING\x10\x00\x12\r\n\tGET_READY\x10\x01\x12\r\n\tLISTENING\x10\x02\x12\x0b\n\x07PLAYING\x10\x03\x12\x0c\n\x08SHIFTING\x10\x04\x12\x0c\n\x08THROW_IN\x10\x05\x12\x0f\n\x0b\x43ORNER_KICK\x10\x06\x12\x0b\n\x07PENALTY\x10\x07\x12\t\n\x05\x46\x41ULT\x10\x08\x12\x08\n\x04OVER\x10\x63\"}\n\x04Team\x12\x1d\n\x07players\x18\x01 \x03(\x0b\x32\x0c.lugo.Player\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05score\x18\x03 \x01(\r\x12\x1d\n\x04side\x18\x04 \x01(\x0e\x32\x0f.lugo.Team.Side\"\x1a\n\x04Side\x12\x08\n\x04HOME\x10\x00\x12\x08\n\x04\x41WAY\x10\x01\"H\n\tShotClock\x12\"\n\tteam_side\x18\x06 \x01(\x0e\x32\x0f.lugo.Team.Side\x12\x17\n\x0fremaining_turns\x18\x07 \x01(\r\"\xb5\x01\n\x06Player\x12\x0e\n\x06number\x18\x01 \x01(\r\x12\x1d\n\x08position\x18\x02 \x01(\x0b\x32\x0b.lugo.Point\x12 \n\x08velocity\x18\x03 \x01(\x0b\x32\x0e.lugo.Velocity\x12\"\n\tteam_side\x18\x04 \x01(\x0e\x32\x0f.lugo.Team.Side\x12\"\n\rinit_position\x18\x05 \x01(\x0b\x32\x0b.lugo.Point\x12\x12\n\nis_jumping\x18\x06 \x01(\x08\"e\n\x04\x42\x61ll\x12\x1d\n\x08position\x18\x01 \x01(\x0b\x32\x0b.lugo.Point\x12 \n\x08velocity\x18\x02 \x01(\x0b\x32\x0e.lugo.Velocity\x12\x1c\n\x06holder\x18\x03 \x01(\x0b\x32\x0c.lugo.Player\"\xab\x01\n\rOrderResponse\x12,\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x1e.lugo.OrderResponse.StatusCode\x12\x0f\n\x07\x64\x65tails\x18\x02 \x01(\t\"[\n\nStatusCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\x12\n\x0eUNKNOWN_PLAYER\x10\x01\x12\x11\n\rNOT_LISTENING\x10\x02\x12\x0e\n\nWRONG_TURN\x10\x03\x12\t\n\x05OTHER\x10\x63\"L\n\x08OrderSet\x12\x0c\n\x04turn\x18\x01 \x01(\r\x12\x1b\n\x06orders\x18\x02 \x03(\x0b\x32\x0b.lugo.Order\x12\x15\n\rdebug_message\x18\x03 \x01(\t\"\x83\x01\n\x05Order\x12\x1a\n\x04move\x18\x01 \x01(\x0b\x32\n.lugo.MoveH\x00\x12\x1c\n\x05\x63\x61tch\x18\x02 \x01(\x0b\x32\x0b.lugo.CatchH\x00\x12\x1a\n\x04kick\x18\x03 \x01(\x0b\x32\n.lugo.KickH\x00\x12\x1a\n\x04jump\x18\x04 \x01(\x0b\x32\n.lugo.JumpH\x00\x42\x08\n\x06\x61\x63tion\"(\n\x04Move\x12 \n\x08velocity\x18\x01 \x01(\x0b\x32\x0e.lugo.Velocity\"\x07\n\x05\x43\x61tch\"(\n\x04Kick\x12 \n\x08velocity\x18\x01 \x01(\x0b\x32\x0e.lugo.Velocity\"(\n\x04Jump\x12 \n\x08velocity\x18\x01 \x01(\x0b\x32\x0e.lugo.Velocity2o\n\x04Game\x12\x34\n\tJoinATeam\x12\x11.lugo.JoinRequest\x1a\x12.lugo.GameSnapshot0\x01\x12\x31\n\nSendOrders\x12\x0e.lugo.OrderSet\x1a\x13.lugo.OrderResponseB#Z!github.com/lugobots/lugo4go/protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'server_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z!github.com/lugobots/lugo4go/proto'
  _JOINREQUEST._serialized_start=38
  _JOINREQUEST._serialized_end=180
  _GAMESNAPSHOT._serialized_start=183
  _GAMESNAPSHOT._serialized_end=555
  _GAMESNAPSHOT_STATE._serialized_start=413
  _GAMESNAPSHOT_STATE._serialized_end=555
  _TEAM._serialized_start=557
  _TEAM._serialized_end=682
  _TEAM_SIDE._serialized_start=656
  _TEAM_SIDE._serialized_end=682
  _SHOTCLOCK._serialized_start=684
  _SHOTCLOCK._serialized_end=756
  _PLAYER._serialized_start=759
  _PLAYER._serialized_end=940
  _BALL._serialized_start=942
  _BALL._serialized_end=1043
  _ORDERRESPONSE._serialized_start=1046
  _ORDERRESPONSE._serialized_end=1217
  _ORDERRESPONSE_STATUSCODE._serialized_start=1126
  _ORDERRESPONSE_STATUSCODE._serialized_end=1217
  _ORDERSET._serialized_start=1219
  _ORDERSET._serialized_end=1295
  _ORDER._serialized_start=1298
  _ORDER._serialized_end=1429
  _MOVE._serialized_start=1431
  _MOVE._serialized_end=1471
  _CATCH._serialized_start=1473
  _CATCH._serialized_end=1480
  _KICK._serialized_start=1482
  _KICK._serialized_end=1522
  _JUMP._serialized_start=1524
  _JUMP._serialized_end=1564
  _GAME._serialized_start=1566
  _GAME._serialized_end=1677
# @@protoc_insertion_point(module_scope)



from ..protos.physics_pb2 import Point
from ..protos.server_pb2 import Team


class Goal(object):
    """
    Represents a goal element in the field a game.

    Attributes:
        _center (lugo.Point): The center point of the goal.
        _place (lugo.TeamSide): The side to which the goal belongs.
        _topPole (lugo.Point): The top pole of the goal.
        _bottomPole (lugo.Point): The bottom pole of the goal.

    Methods:
        get_center() -> Point: Get the center point of the goal.
        get_place() -> Team.Side: Get the side to which the goal belongs.
        get_top_pole() -> Point: Get the top pole point of the goal.
        get_bottom_pole() -> Point: Get the bottom pole point of the goal.

    Usage:
    goal = Goal(place, center, top_pole, bottom_pole)
    center_point = goal.get_center()
    goal_side = goal.get_place()
    top_pole_point = goal.get_top_pole()
    bottom_pole_point = goal.get_bottom_pole()
    """
    def __init__(self, place: Team.Side, center: Point, top_pole: Point, bottom_pole: Point):
        self._center = center
        self._place = place
        self._topPole = top_pole
        self._bottomPole = bottom_pole

    def get_center(self) -> Point:
        """
        Get the center point of the goal.

        Returns:
            Point: The center point of the goal.
        """
        return self._center

    def get_place(self) -> Team.Side:
        """
        Get the side to which the goal belongs.

        Returns:
            Team.Side: The side of the team to which the goal belongs.
        """
        return self._place

    def get_top_pole(self) -> Point:
        """
        Get the top pole point of the goal.

        Returns:
            Point: The top pole point of the goal.
        """
        return self._topPole

    def get_bottom_pole(self) -> Point:
        """
        Get the bottom pole point of the goal.

        Returns:
            Point: The bottom pole point of the goal.
        """
        return self._bottomPole



from .. import Point
from ..mapper import Mapper
from .utils.defaults import DefaultInitBundle
from .loader import EnvVarLoader
from .interface import Bot
from .client import RawTurnProcessor, NewClientFromConfig
from concurrent.futures import ThreadPoolExecutor
from typing import Callable
import signal
import sys

def NewDefaultStarter():
    defaultConfig, defaultMapper, defaultInitialPosition = DefaultInitBundle()
    return Starter(defaultInitialPosition, defaultConfig, defaultMapper)

class Starter:
    def __init__(self, initial_position: Point, config: EnvVarLoader, mapper: Mapper):
        self.initial_position = initial_position
        self.config = config
        self.mapper = mapper

    def get_mapper(self):
        return self.mapper

    def set_mapper(self, mapper: Mapper):
        self.mapper = mapper

    def get_initial_position(self):
        return self.initial_position

    def set_initial_position(self, initial_position: Point):
        self.initial_position = initial_position

    def get_config(self):
        return self.config

    def set_config(self, config: EnvVarLoader):
        self.config = config

    def run(self, bot: Bot, on_join: Callable[[], None]):
        lugoClient = NewClientFromConfig(self.config, self.initial_position)
        executor = ThreadPoolExecutor()
        lugoClient.play_as_bot(executor, bot, on_join)
        
        print("We are playing!")
        def signal_handler(_, __):
            print("Stop requested\n")
            lugoClient.stop()
            sys.exit(0)

        signal.signal(signal.SIGINT, signal_handler)
        lugoClient.wait()
        print("bye!")

    def run_just_turn_handler(self, raw_processor: RawTurnProcessor, on_join: Callable[[], None]):
        lugoClient = NewClientFromConfig(self.config, self.initial_position)
        lugoClient.play(raw_processor, on_join).then(lambda: print("all done")).catch(lambda e: print(e))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import rl_assistant_pb2 as rl__assistant__pb2
import server_pb2 as server__pb2


class RLAssistantStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartSession = channel.unary_stream(
                '/lugo.RLAssistant/StartSession',
                request_serializer=rl__assistant__pb2.RLSessionConfig.SerializeToString,
                response_deserializer=server__pb2.GameSnapshot.FromString,
                )
        self.SendPlayersOrders = channel.unary_unary(
                '/lugo.RLAssistant/SendPlayersOrders',
                request_serializer=rl__assistant__pb2.PlayersOrders.SerializeToString,
                response_deserializer=rl__assistant__pb2.TurnOutcome.FromString,
                )
        self.ResetEnv = channel.unary_unary(
                '/lugo.RLAssistant/ResetEnv',
                request_serializer=rl__assistant__pb2.RLResetConfig.SerializeToString,
                response_deserializer=rl__assistant__pb2.TurnOutcome.FromString,
                )


class RLAssistantServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StartSession(self, request, context):
        """StartSession will create 11 player connections on each team and wait the game to be on the waiting status
        It will return a stream of game snapshots, like "Game.JoinATeam" does, but these GameSnapshot will be the
        same one returned by SendPlayersOrders. But you need to keep the stream open, otherwise the server will
        consider that the session is done.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayersOrders(self, request, context):
        """SendPlayersOrders will send all payers orders, and ignore the ones that do not have orders
        If there is no order from a player, it will behave are defined in PlayersOrders.default_behaviour.
        The valid values for default_behaviour are defined by the server. Please run the server help command
        for more information.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetEnv(self, request, context):
        """The training environment will need to reset the game status multiple times during the training session.
        Each time this occurs, the RL assistant must be notified by the trainer, as the game server does not emit any
        related events.
        When the environment is reset, the RL assistant will also reset its controllers to maintain synchronization with
        the game server.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RLAssistantServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartSession': grpc.unary_stream_rpc_method_handler(
                    servicer.StartSession,
                    request_deserializer=rl__assistant__pb2.RLSessionConfig.FromString,
                    response_serializer=server__pb2.GameSnapshot.SerializeToString,
            ),
            'SendPlayersOrders': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayersOrders,
                    request_deserializer=rl__assistant__pb2.PlayersOrders.FromString,
                    response_serializer=rl__assistant__pb2.TurnOutcome.SerializeToString,
            ),
            'ResetEnv': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetEnv,
                    request_deserializer=rl__assistant__pb2.RLResetConfig.FromString,
                    response_serializer=rl__assistant__pb2.TurnOutcome.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'lugo.RLAssistant', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class RLAssistant(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StartSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/lugo.RLAssistant/StartSession',
            rl__assistant__pb2.RLSessionConfig.SerializeToString,
            server__pb2.GameSnapshot.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendPlayersOrders(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lugo.RLAssistant/SendPlayersOrders',
            rl__assistant__pb2.PlayersOrders.SerializeToString,
            rl__assistant__pb2.TurnOutcome.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetEnv(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lugo.RLAssistant/ResetEnv',
            rl__assistant__pb2.RLResetConfig.SerializeToString,
            rl__assistant__pb2.TurnOutcome.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import contextlib
import traceback
from concurrent.futures import ThreadPoolExecutor

import grpc
import time
from typing import Any, Iterator

from src.lugo4py import GameSnapshot
from src.lugo4py.protos.remote_pb2_grpc import Remote, RemoteStub
from src.lugo4py.protos.rl_assistant_pb2 import RLSessionConfig
from src.lugo4py.protos.rl_assistant_pb2_grpc import RLAssistantStub
from src.lugo4py.rl.src.training_controller import TrainingCrl
from src.lugo4py.rl.src.contracts import BotTrainer, TrainingFunction
from src.lugo4py.src.client import log_with_time


class Gym:

    def __init__(self, executor: ThreadPoolExecutor,  grpc_address):

        channel = grpc.insecure_channel(grpc_address)

        # Block until channel is ready (or timeout)
        grpc.channel_ready_future(channel).result(timeout=5)

        self.remote = RemoteStub(channel)
        self.assistant = RLAssistantStub(channel)
        self.executor = executor


    def start(self, trainer: BotTrainer, training_function: TrainingFunction) -> None:

        training_ctrl = TrainingCrl(trainer, self.remote, self.assistant)

        response_iterator = self.assistant.StartSession(request=RLSessionConfig())


        self._training_routine = self.executor.submit(self._response_watcher, response_iterator)


        time.sleep(2)
        training_function(training_ctrl)

    def _response_watcher(
        self,
        response_iterator: Iterator[GameSnapshot],
        ) -> None:
        try:
            for snapshot in response_iterator:
                log_with_time(f"still alive")

            # self._play_finished.set()
        except grpc.RpcError as e:
            if grpc.StatusCode.INVALID_ARGUMENT == e.code():
                log_with_time(f"did not connect to RL session {e.details()}")
        except Exception as e:
            log_with_time(f"internal error processing RL session: {e}")
            traceback.print_exc()



"""
File: lugo.py
Author: Angelo Katipunan
Date: May 20, 2023
Description: This file mocks the gRPC methods to help IDEs intellisense.

Python gRPC files are not friendly to IDEs, what makes the intellisense experience very poor or, sometimes, impossible.

In order to help the programmer experience while developing bots, this file mocks the gRPC methods in a more friendly way.

In short, this file content is not used at all by the package, but will guide the IDE to help the devs.
If you are looking for the real implementation of these methods, please look at the `protos` directory (good luck on that)

"""

from enum import IntEnum
from typing import List

from ..protos.physics_pb2 import Velocity, Vector
from ..protos.server_pb2 import Team

def new_velocity(vector: Vector) -> Velocity:
    """
    Create a new Velocity object based on a Vector.

    This function takes a Vector object and creates a new Velocity object by setting its direction
    components based on the x and y components from the provided Vector.

    Args:
        vector (Vector): A Vector object representing the direction components.

    Returns:
        Velocity: A new Velocity object with the direction components set from the Vector.

    Example:
    vector = Vector(1.0, 0.0)
    velocity = new_velocity(vector)
    """
    v = Velocity()
    v.direction.x = vector.x
    v.direction.y = vector.y
    return v



class TeamSide(IntEnum):
    HOME = Team.Side.HOME
    AWAY = Team.Side.AWAY




from ..protos.physics_pb2 import Point
from ..protos.server_pb2 import Order, Team
from .game_snapshot_inspector import GameSnapshotInspector
from .define_state import PLAYER_STATE
from ..mapper import Mapper
from abc import ABC, abstractmethod
from typing import List

class Bot(ABC):
    """
    Abstract base class representing a bot in a game.

    Attributes:
        side (TeamSide): The side to which the bot belongs.
        number (int): The player number in its team
        init_position (Point): The initial position of the bot.
        mapper (Mapper): The mapper associated with the bot.

    Methods:
        on_disputing(order_set: OrderSet, game_snapshot: GameSnapshot) -> OrderSet: Method called when the player is on DISPUTING_THE_BALL state
        on_defending(order_set: OrderSet, game_snapshot: GameSnapshot) -> OrderSet: Method called when the player is on DEFENDING state
        on_holding(order_set: OrderSet, game_snapshot: GameSnapshot) -> OrderSet: Method called when the player is on HOLDING_THE_BALL state
        on_supporting(order_set: OrderSet, game_snapshot: GameSnapshot) -> OrderSet: Method called when the player is on SUPPORTING state
        as_goalkeeper(order_set: OrderSet, game_snapshot: GameSnapshot, state: PLAYER_STATE) -> OrderSet: Method is called on every turn, and the player state is passed at the last parameter.
        getting_ready(game_snapshot: GameSnapshot): Abstract method for bot preparation before the game.

    Usage:
    Define a subclass of Bot and implement the abstract methods for specific bot behaviors.
    """
    def __init__(self, side: Team.Side, number: int, init_position: Point, my_mapper: Mapper):
        self.number = number
        self.side = side
        self.mapper = my_mapper
        self.initPosition = init_position

    @abstractmethod
    def on_disputing(self, inspector: GameSnapshotInspector) -> List[Order]:
        """
        Method called when the player is on DISPUTING_THE_BALL state.

        Args:
            order_set (OrderSet): The current set of orders.
            game_snapshot (GameSnapshot): The current game snapshot.

        Returns:
            OrderSet: The updated set of orders.
        """
        pass

    @abstractmethod
    def on_defending(self, inspector: GameSnapshotInspector) -> List[Order]:
        """
        Method called when the player is on DEFENDING state

        Args:
            order_set (OrderSet): The current set of orders.
            game_snapshot (GameSnapshot): The current game snapshot.

        Returns:
            OrderSet: The updated set of orders.
        """
        pass

    @abstractmethod
    def on_holding(self, inspector: GameSnapshotInspector) -> List[Order]:
        """
        Method called when the player is on HOLDING_THE_BALL state

        Args:
            order_set (OrderSet): The current set of orders.
            game_snapshot (GameSnapshot): The current game snapshot.

        Returns:
            OrderSet: The updated set of orders.
        """
        pass

    @abstractmethod
    def on_supporting(self, inspector: GameSnapshotInspector) -> List[Order]:
        """
        Method called when the player is on SUPPORTING state

        Args:
            order_set (OrderSet): The current set of orders.
            game_snapshot (GameSnapshot): The current game snapshot.

        Returns:
            OrderSet: The updated set of orders.
        """
        pass

    @abstractmethod
    def as_goalkeeper(self, inspector: GameSnapshotInspector, state: PLAYER_STATE) -> List[Order]:
        """
        Method is called on every turn, and the player state is passed at the last parameter.

        Args:
            order_set (OrderSet): The current set of orders.
            game_snapshot (GameSnapshot): The current game snapshot.
            state (PLAYER_STATE): The current state of the bot.

        Returns:
            OrderSet: The updated set of orders.
        """
        pass

    @abstractmethod
    def getting_ready(self, inspector: GameSnapshotInspector):
        """
        Method called before the game starts and right after the score is changed

        Args:
            game_snapshot (GameSnapshot): The current game snapshot.
        """
        pass

    # def make_reader(self, game_snapshot: lugo.GameSnapshot) -> Tuple[snapshot.GameSnapshotReader, lugo.Player]:
    #     """
    #     Create a game snapshot reader for the bot's side and retrieve the bot's player information.

    #     Args:
    #         game_snapshot (GameSnapshot): The current game snapshot.

    #     Returns:
    #         snapshot.GameSnapshotReader: The game snapshot reader for the bot's side.
    #         Player: The bot's player information.

    #     Raises:
    #         AttributeError: If the bot is not found in the game snapshot.
    #     """
    #     reader = snapshot.GameSnapshotReader(game_snapshot, self.side)
    #     me = reader.get_player(self.side, self.number)
    #     if me is None:
    #         raise AttributeError("did not find myself in the game")

    #     return reader, me






from .src.client import PROTOCOL_VERSION
from .src.client import LugoClient
from .src.client import NewClientFromConfig

from .src.geo import new_vector, sub_vector, is_invalid_vector, get_scaled_vector, get_length, distance_between_points, \
    normalize
from .src.goal import Goal

from .src.interface import Bot

from .src.define_state import PLAYER_STATE, PlayerState

from .src.loader import EnvVarLoader

from .src.lugo import new_velocity
from .mapper import *

from .src.game_snapshot_inspector import *

from .src.specs import *

from .src.starter import *

from .src.utils.defaults import *

from .protos.server_pb2 import *
from .protos.server_pb2_grpc import *
from .protos.physics_pb2 import *
from .protos.physics_pb2_grpc import *
from .protos.remote_pb2 import *
from .protos.remote_pb2_grpc import *
from .protos.broadcast_pb2 import *
from .protos.broadcast_pb2_grpc import *



from ..protos import server_pb2
from ..mapper import DIRECTION, ORIENTATION, homeGoal, awayGoal

from . import geo, helpers, specs
from ..protos.physics_pb2 import Point, Vector
from ..protos.server_pb2 import Team, GameSnapshot, Player


# Assuming that Lugo, Geo, Helpers, and other dependencies are defined elsewhere

class GameSnapshotInspector:
    def __init__(self, bot_side: Team.Side, player_number: int, game_snapshot: GameSnapshot):
        self.my_side = bot_side
        self.my_number = player_number
        self.snapshot = game_snapshot
        self.me = self.get_player(bot_side, player_number)
        if not self.me:
            raise ValueError(f"Could not find the player {bot_side}-{player_number}")

    def get_snapshot(self):
        return self.snapshot

    def get_turn(self):
        return self.snapshot.turn

    def get_me(self):
        return self.me

    def get_ball(self):
        return self.snapshot.ball if self.snapshot else None

    def get_player(self, side: Team.Side, number: int):
        return helpers.get_player(self.snapshot, side, number)

    def get_ball_holder(self):
        return helpers.get_ball_holder(self.snapshot)

    def is_ball_holder(self, player: Player):
        return helpers.is_ball_holder(self.snapshot, player)

    def get_team(self, side: Team.Side):
        return helpers.get_team(self.snapshot, side)

    def get_my_team(self):
        return self.get_team(self.my_side)

    def get_opponent_team(self):
        return self.get_team(self.get_opponent_side())

    def get_my_team_side(self):
        return self.my_side

    def get_opponent_side(self):
        return helpers.get_opponent_side(self.my_side)

    def get_my_team_players(self):
        my_team = self.get_my_team()
        return my_team.players if my_team else []

    def get_opponent_players(self):
        opponent_team = self.get_opponent_team()
        return opponent_team.players if opponent_team else []

    def get_my_team_goalkeeper(self):
        return self.get_player(self.get_my_team_side(), specs.GOALKEEPER_NUMBER)

    def get_opponent_goalkeeper(self):
        return self.get_player(self.get_opponent_side(), specs.GOALKEEPER_NUMBER)

    # ###############

    def make_order_move(self, target: Point, speed: int):
        return self.make_order_move_from_point(self.me.position if self.me else geo.new_zeroed_point(), target, speed)

    def make_order_move_max_speed(self, target: Point):
        return self.make_order_move_from_point(self.me.position if self.me else geo.new_zeroed_point(), target, specs.PLAYER_MAX_SPEED)

    def make_order_move_from_point(self, origin: Point, target: Point, speed: int):
        direction = geo.new_vector(origin, target)
        normalizedDirection = geo.normalize(direction)

        order = server_pb2.Order()
        order.move.velocity.direction.CopyFrom(normalizedDirection)
        order.move.velocity.speed = speed
        return order

    def make_order_move_from_vector(self, direction: Vector, speed: int):
        target_point = geo.target_from(direction, self.me.position if self.me else geo.new_zeroed_point())
        return self.make_order_move_from_point(self.me.position if self.me else geo.new_zeroed_point(), target_point, speed)

    def make_order_move_by_direction(self, direction, speed=None):
        direction_target = self.get_orientation_by_direction(direction, self.my_side)
        return self.make_order_move_from_vector(direction_target, speed if speed is not None else specs.PLAYER_MAX_SPEED)

    def make_order_move_to_stop(self):
        my_direction = self.me.velocity.direction if self.me and self.me.velocity else self.get_orientation_by_direction(DIRECTION.FORWARD)
        return self.make_order_move_from_vector(my_direction, 0)

    def make_order_jump(self, target, speed):
        direction = geo.new_vector(self.me.position if self.me else geo.new_zeroed_point(), target)
        normalizedDirection = geo.normalize(direction)

        order = server_pb2.Order()
        order.jump.velocity.direction.CopyFrom(normalizedDirection)
        order.jump.velocity.speed = speed
        return order

    def make_order_kick(self, target: Point, speed: int):
        ball_expected_direction = geo.new_vector(self.get_ball().position if self.snapshot and self.get_ball() else geo.new_zeroed_point(), target)
        diff_vector = geo.sub_vector(ball_expected_direction, self.get_ball().velocity.direction if self.snapshot and self.get_ball() and self.get_ball().velocity else geo.new_zeroed_point())
        normalizedDirection = geo.normalize(diff_vector)

        order = server_pb2.Order()
        order.kick.velocity.direction.CopyFrom(normalizedDirection)
        order.kick.velocity.speed = speed
        return order

    def make_order_kick_max_speed(self, target: Point):
        return self.make_order_kick(target, specs.BALL_MAX_SPEED)

    def make_order_catch(self):
        order = server_pb2.Order()
        order.catch.SetInParent()
        return order

    def get_orientation_by_direction(self, direction: DIRECTION, my_side: Team.Side):
        if direction == DIRECTION.FORWARD:
            return ORIENTATION.EAST if my_side == Team.Side.HOME else ORIENTATION.WEST
        elif direction == DIRECTION.BACKWARD:
            return ORIENTATION.WEST if my_side == Team.Side.HOME else ORIENTATION.EAST
        elif direction == DIRECTION.LEFT:
            return ORIENTATION.NORTH if my_side == Team.Side.HOME else ORIENTATION.SOUTH
        elif direction == DIRECTION.RIGHT:
            return ORIENTATION.SOUTH if my_side == Team.Side.HOME else ORIENTATION.NORTH
        elif direction == DIRECTION.BACKWARD_LEFT:
            return ORIENTATION.NORTH_WEST if my_side == Team.Side.HOME else ORIENTATION.SOUTH_EAST
        elif direction == DIRECTION.BACKWARD_RIGHT:
            return ORIENTATION.SOUTH_WEST if my_side == Team.Side.HOME else ORIENTATION.NORTH_EAST
        elif direction == DIRECTION.FORWARD_LEFT:
            return ORIENTATION.NORTH_EAST if my_side == Team.Side.HOME else ORIENTATION.SOUTH_WEST
        elif direction == DIRECTION.FORWARD_RIGHT:
            return ORIENTATION.SOUTH_EAST if my_side == Team.Side.HOME else ORIENTATION.NORTH_WEST
        else:
            raise ValueError(f"Unknown direction {direction}")


import contextlib
import grpc
import time
from typing import Any

from src.lugo4py.protos.remote_pb2_grpc import Remote
from src.lugo4py.protos.rl_assistant_pb2 import RLResetConfig
from src.lugo4py.protos.rl_assistant_pb2_grpc import RLAssistant
from src.lugo4py.rl.src.contracts import BotTrainer


class TrainingCrl:
    def __init__(self, trainer: BotTrainer, remote: Remote, assistant: RLAssistant):
        self.remote = remote
        self.assistant = assistant
        self.trainer = trainer
        self.latest_snapshot = None

    def set_environment(self, data: Any) -> None:
        try:
            self.latest_snapshot = self.trainer.create_new_initial_state(data)
        except Exception as e:
            raise RuntimeError(f"Failed to set environment: {e}")

        try:
            # self.assistant.ResetEnv(RLResetConfig(), "localhost:5000")
            self.assistant.ResetEnv(request=RLResetConfig())
        except grpc.RpcError as e:
            raise RuntimeError(f"Failed to reset RL assistant: {e}")

    def get_state(self) -> Any:
        return self.trainer.get_training_state(self.latest_snapshot)

    def update(self, action: Any) -> tuple[float, bool]:
        try:
            players_orders = self.trainer.play(self.latest_snapshot, action)
        except Exception as e:
            raise RuntimeError(f"Trainer bot failed to play: {e}")

        try:
            turn_outcome = self.assistant.SendPlayersOrders(players_orders)
        except grpc.RpcError as e:
            raise RuntimeError(f"RL assistant failed to send the orders: {e}")

        previous_snapshot = self.latest_snapshot
        self.latest_snapshot = turn_outcome.game_snapshot
        return self.trainer.evaluate(previous_snapshot, turn_outcome.game_snapshot, turn_outcome)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: broadcast.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import server_pb2 as server__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x62roadcast.proto\x12\x04lugo\x1a\x0cserver.proto\"\x1e\n\x0eWatcherRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\t\"$\n\x0cStartRequest\x12\x14\n\x0cwatcher_uuid\x18\x01 \x01(\t\"\xfa\x02\n\tGameEvent\x12)\n\rgame_snapshot\x18\x01 \x01(\x0b\x32\x12.lugo.GameSnapshot\x12*\n\nnew_player\x18\x02 \x01(\x0b\x32\x14.lugo.EventNewPlayerH\x00\x12,\n\x0blost_player\x18\x03 \x01(\x0b\x32\x15.lugo.EventLostPlayerH\x00\x12.\n\x0cstate_change\x18\x04 \x01(\x0b\x32\x16.lugo.EventStateChangeH\x00\x12\x1f\n\x04goal\x18\x05 \x01(\x0b\x32\x0f.lugo.EventGoalH\x00\x12(\n\tgame_over\x18\x06 \x01(\x0b\x32\x13.lugo.EventGameOverH\x00\x12\x30\n\nbreakpoint\x18\x07 \x01(\x0b\x32\x1a.lugo.EventDebugBreakpointH\x00\x12\x32\n\x0e\x64\x65\x62ug_released\x18\x08 \x01(\x0b\x32\x18.lugo.EventDebugReleasedH\x00\x42\x07\n\x05\x65vent\"\xfa\x02\n\tGameSetup\x12\x18\n\x10protocol_version\x18\x01 \x01(\t\x12\x10\n\x08\x64\x65v_mode\x18\x02 \x01(\x08\x12\x30\n\nstart_mode\x18\x03 \x01(\x0e\x32\x1c.lugo.GameSetup.StartingMode\x12\x35\n\x0elistening_mode\x18\x04 \x01(\x0e\x32\x1d.lugo.GameSetup.ListeningMode\x12\x1a\n\x12listening_duration\x18\x05 \x01(\r\x12\x15\n\rgame_duration\x18\x06 \x01(\r\x12%\n\thome_team\x18\x08 \x01(\x0b\x32\x12.lugo.TeamSettings\x12%\n\taway_team\x18\t \x01(\x0b\x32\x12.lugo.TeamSettings\"%\n\x0cStartingMode\x12\x0b\n\x07NO_WAIT\x10\x00\x12\x08\n\x04WAIT\x10\x01\"0\n\rListeningMode\x12\t\n\x05TIMER\x10\x00\x12\x08\n\x04RUSH\x10\x01\x12\n\n\x06REMOTE\x10\x02\"N\n\x0cTeamSettings\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06\x61vatar\x18\x02 \x01(\t\x12 \n\x06\x63olors\x18\x03 \x01(\x0b\x32\x10.lugo.TeamColors\"R\n\nTeamColors\x12 \n\x07primary\x18\x01 \x01(\x0b\x32\x0f.lugo.TeamColor\x12\"\n\tsecondary\x18\x02 \x01(\x0b\x32\x0f.lugo.TeamColor\"5\n\tTeamColor\x12\x0b\n\x03red\x18\x01 \x01(\r\x12\r\n\x05green\x18\x02 \x01(\r\x12\x0c\n\x04\x62lue\x18\x04 \x01(\r\".\n\x0e\x45ventNewPlayer\x12\x1c\n\x06player\x18\x01 \x01(\x0b\x32\x0c.lugo.Player\"/\n\x0f\x45ventLostPlayer\x12\x1c\n\x06player\x18\x01 \x01(\x0b\x32\x0c.lugo.Player\"q\n\x10\x45ventStateChange\x12\x30\n\x0eprevious_state\x18\x01 \x01(\x0e\x32\x18.lugo.GameSnapshot.State\x12+\n\tnew_state\x18\x02 \x01(\x0e\x32\x18.lugo.GameSnapshot.State\"*\n\tEventGoal\x12\x1d\n\x04side\x18\x01 \x01(\x0e\x32\x0f.lugo.Team.Side\"\xa7\x02\n\rEventGameOver\x12\x30\n\x06reason\x18\x01 \x01(\x0e\x32 .lugo.EventGameOver.EndingReason\x12,\n\x05\x62lame\x18\x02 \x01(\x0e\x32\x1d.lugo.EventGameOver.BlameStop\"o\n\x0c\x45ndingReason\x12\x10\n\x0cTIME_IS_OVER\x10\x00\x12\x13\n\x0fWAITING_EXPIRED\x10\x01\x12\x14\n\x10NO_ENOUGH_PLAYER\x10\x02\x12\x14\n\x10\x45XTERNAL_REQUEST\x10\x03\x12\x0c\n\x08KNOCKOUT\x10\x04\"E\n\tBlameStop\x12\n\n\x06NORMAL\x10\x00\x12\x0e\n\nBOTH_TEAMS\x10\x01\x12\r\n\tHOME_TEAM\x10\x02\x12\r\n\tAWAY_TEAM\x10\x03\"u\n\x14\x45ventDebugBreakpoint\x12\x39\n\nbreakpoint\x18\x01 \x01(\x0e\x32%.lugo.EventDebugBreakpoint.Breakpoint\"\"\n\nBreakpoint\x12\n\n\x06ORDERS\x10\x00\x12\x08\n\x04TURN\x10\x01\"\x14\n\x12\x45ventDebugReleased2\xa8\x01\n\tBroadcast\x12\x32\n\x07OnEvent\x12\x14.lugo.WatcherRequest\x1a\x0f.lugo.GameEvent0\x01\x12\x35\n\x0cGetGameSetup\x12\x14.lugo.WatcherRequest\x1a\x0f.lugo.GameSetup\x12\x30\n\tStartGame\x12\x12.lugo.StartRequest\x1a\x0f.lugo.GameSetupB#Z!github.com/lugobots/lugo4go/protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'broadcast_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z!github.com/lugobots/lugo4go/proto'
  _WATCHERREQUEST._serialized_start=39
  _WATCHERREQUEST._serialized_end=69
  _STARTREQUEST._serialized_start=71
  _STARTREQUEST._serialized_end=107
  _GAMEEVENT._serialized_start=110
  _GAMEEVENT._serialized_end=488
  _GAMESETUP._serialized_start=491
  _GAMESETUP._serialized_end=869
  _GAMESETUP_STARTINGMODE._serialized_start=782
  _GAMESETUP_STARTINGMODE._serialized_end=819
  _GAMESETUP_LISTENINGMODE._serialized_start=821
  _GAMESETUP_LISTENINGMODE._serialized_end=869
  _TEAMSETTINGS._serialized_start=871
  _TEAMSETTINGS._serialized_end=949
  _TEAMCOLORS._serialized_start=951
  _TEAMCOLORS._serialized_end=1033
  _TEAMCOLOR._serialized_start=1035
  _TEAMCOLOR._serialized_end=1088
  _EVENTNEWPLAYER._serialized_start=1090
  _EVENTNEWPLAYER._serialized_end=1136
  _EVENTLOSTPLAYER._serialized_start=1138
  _EVENTLOSTPLAYER._serialized_end=1185
  _EVENTSTATECHANGE._serialized_start=1187
  _EVENTSTATECHANGE._serialized_end=1300
  _EVENTGOAL._serialized_start=1302
  _EVENTGOAL._serialized_end=1344
  _EVENTGAMEOVER._serialized_start=1347
  _EVENTGAMEOVER._serialized_end=1642
  _EVENTGAMEOVER_ENDINGREASON._serialized_start=1460
  _EVENTGAMEOVER_ENDINGREASON._serialized_end=1571
  _EVENTGAMEOVER_BLAMESTOP._serialized_start=1573
  _EVENTGAMEOVER_BLAMESTOP._serialized_end=1642
  _EVENTDEBUGBREAKPOINT._serialized_start=1644
  _EVENTDEBUGBREAKPOINT._serialized_end=1761
  _EVENTDEBUGBREAKPOINT_BREAKPOINT._serialized_start=1727
  _EVENTDEBUGBREAKPOINT_BREAKPOINT._serialized_end=1761
  _EVENTDEBUGRELEASED._serialized_start=1763
  _EVENTDEBUGRELEASED._serialized_end=1783
  _BROADCAST._serialized_start=1786
  _BROADCAST._serialized_end=1954
# @@protoc_insertion_point(module_scope)



from typing import Any, Tuple, Protocol, Callable

from src.lugo4py import Team, Order
from src.lugo4py.protos.rl_assistant_pb2 import PlayersOrders, TurnOutcome
from src.lugo4py.protos.server_pb2 import GameSnapshot


class TrainingController(Protocol):
    def set_environment(self, data: Any) -> None:
        """Resets the game to an initial state, passing any necessary data."""
        pass

    def get_state(self) -> Any:
        """Retrieves the inputs used by the model, e.g., tensors for neural networks."""
        pass

    def update(self, action: Any) -> Tuple[float, bool]:
        """Passes the action picked by the model and returns the reward and done values."""
        pass


class BotTrainer(Protocol):
    def create_new_initial_state(self, data: Any) -> GameSnapshot:
        """Sets up the initial state for each game."""
        pass

    def get_training_state(self, snapshot: GameSnapshot) -> Any:
        """Returns the input values (e.g., sensor data) based on the current game state."""
        pass

    def play(self, game_snapshot: GameSnapshot, action: Any) -> PlayersOrders:
        """Translates the action chosen by the model to game orders."""
        pass

    def evaluate(self, previous_game_snapshot: GameSnapshot, new_game_snapshot: GameSnapshot, turn_outcome: TurnOutcome) -> Tuple[float, bool]:
        """Compares the previous and new game states to determine the reward and whether the game is done."""
        pass


class PlayersOrdersBuilder(Protocol):
    def add_order(self, player_number, team_side: Team.Side, orders: list[Order]) -> "PlayersOrdersBuilder":
        pass

    def set_player_behaviour(self, player_number, team_side: Team.Side, behaviour) -> "PlayersOrdersBuilder":
        pass

    def build(self) -> PlayersOrders:
        pass


TrainingFunction = Callable[[TrainingController], None]


class Config:
    def __init__(self, grpc_address):
        """Configuration for the gRPC server."""
        self.grpc_address = grpc_address


# Constants
BOT_BEHAVIOUR_STATUES = "statues"
BOT_BEHAVIOUR_KIDS = "kids"
BOT_BEHAVIOUR_DEFENSES = "defenses"
